@@ -1,7 +1,7 @@
 /*
  *  This is the gdb i386 remote debug stub from gdb 4.XX.
  *
- *  $Id: i386-stub.c,v 1.8 2005/05/23 21:56:38 joel Exp $
+ *  $Id$
  */
 
 /****************************************************************************
@@ -99,6 +99,7 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <stdbool.h>
 
 /************************************************************************
  *
@@ -115,12 +116,12 @@ extern void exceptionHandler (int, void (*handler) (void));
 /* at least NUMREGBYTES*2 are needed for register packets */
 #define BUFMAX 400
 
-static char initialized;	/* boolean flag. != 0 means we've been initialized */
+static bool initialized = false;	/* boolean flag. != 0 means we've been initialized */
 
-int remote_debug;
+extern int remote_debug;
 /*  debug >  0 prints ill-formed commands in valid packets & checksum errors */
 
-void waitabit ();
+extern void waitabit (void);
 
 static const char hexchars[] = "0123456789abcdef";
 
@@ -275,7 +276,7 @@ asm ("iret");
  *   old eip
  *
  */
-extern void _catchException3 ();
+extern void _catchException3 (void);
 asm (".text");
 asm (".globl _catchException3");
 asm ("_catchException3:");
@@ -285,7 +286,7 @@ asm ("pushl $3");
 CALL_HOOK ();
 
 /* Same thing for exception 1.  */
-extern void _catchException1 ();
+extern void _catchException1 (void);
 asm (".text");
 asm (".globl _catchException1");
 asm ("_catchException1:");
@@ -295,7 +296,7 @@ asm ("pushl $1");
 CALL_HOOK ();
 
 /* Same thing for exception 0.  */
-extern void _catchException0 ();
+extern void _catchException0 (void);
 asm (".text");
 asm (".globl _catchException0");
 asm ("_catchException0:");
@@ -305,7 +306,7 @@ asm ("pushl $0");
 CALL_HOOK ();
 
 /* Same thing for exception 4.  */
-extern void _catchException4 ();
+extern void _catchException4 (void);
 asm (".text");
 asm (".globl _catchException4");
 asm ("_catchException4:");
@@ -315,7 +316,7 @@ asm ("pushl $4");
 CALL_HOOK ();
 
 /* Same thing for exception 5.  */
-extern void _catchException5 ();
+extern void _catchException5 (void);
 asm (".text");
 asm (".globl _catchException5");
 asm ("_catchException5:");
@@ -325,7 +326,7 @@ asm ("pushl $5");
 CALL_HOOK ();
 
 /* Same thing for exception 6.  */
-extern void _catchException6 ();
+extern void _catchException6 (void);
 asm (".text");
 asm (".globl _catchException6");
 asm ("_catchException6:");
@@ -335,7 +336,7 @@ asm ("pushl $6");
 CALL_HOOK ();
 
 /* Same thing for exception 7.  */
-extern void _catchException7 ();
+extern void _catchException7 (void);
 asm (".text");
 asm (".globl _catchException7");
 asm ("_catchException7:");
@@ -345,7 +346,7 @@ asm ("pushl $7");
 CALL_HOOK ();
 
 /* Same thing for exception 8.  */
-extern void _catchException8 ();
+extern void _catchException8 (void);
 asm (".text");
 asm (".globl _catchException8");
 asm ("_catchException8:");
@@ -356,7 +357,7 @@ asm ("pushl $8");
 CALL_HOOK ();
 
 /* Same thing for exception 9.  */
-extern void _catchException9 ();
+extern void _catchException9 (void);
 asm (".text");
 asm (".globl _catchException9");
 asm ("_catchException9:");
@@ -366,7 +367,7 @@ asm ("pushl $9");
 CALL_HOOK ();
 
 /* Same thing for exception 10.  */
-extern void _catchException10 ();
+extern void _catchException10 (void);
 asm (".text");
 asm (".globl _catchException10");
 asm ("_catchException10:");
@@ -377,7 +378,7 @@ asm ("pushl $10");
 CALL_HOOK ();
 
 /* Same thing for exception 12.  */
-extern void _catchException12 ();
+extern void _catchException12 (void);
 asm (".text");
 asm (".globl _catchException12");
 asm ("_catchException12:");
@@ -388,7 +389,7 @@ asm ("pushl $12");
 CALL_HOOK ();
 
 /* Same thing for exception 16.  */
-extern void _catchException16 ();
+extern void _catchException16 (void);
 asm (".text");
 asm (".globl _catchException16");
 asm ("_catchException16:");
@@ -400,7 +401,7 @@ CALL_HOOK ();
 /* For 13, 11, and 14 we have to deal with the CHECK_FAULT stuff.  */
 
 /* Same thing for exception 13.  */
-extern void _catchException13 ();
+extern void _catchException13 (void);
 asm (".text");
 asm (".globl _catchException13");
 asm ("_catchException13:");
@@ -412,7 +413,7 @@ asm ("pushl $13");
 CALL_HOOK ();
 
 /* Same thing for exception 11.  */
-extern void _catchException11 ();
+extern void _catchException11 (void);
 asm (".text");
 asm (".globl _catchException11");
 asm ("_catchException11:");
@@ -424,7 +425,7 @@ asm ("pushl $11");
 CALL_HOOK ();
 
 /* Same thing for exception 14.  */
-extern void _catchException14 ();
+extern void _catchException14 (void);
 asm (".text");
 asm (".globl _catchException14");
 asm ("_catchException14:");
@@ -439,6 +440,7 @@ CALL_HOOK ();
  * remcomHandler is a front end for handle_exception.  It moves the
  * stack pointer into an area reserved for debugger use.
  */
+extern void remcomHandler (void);
 asm ("_remcomHandler:");
 asm ("           popl %eax");	/* pop off return address     */
 asm ("           popl %eax");	/* get the exception number   */
@@ -930,8 +932,6 @@ handle_exception (int exceptionVector)
 void
 set_debug_traps (void)
 {
-  extern void remcomHandler ();
-
   stackPtr = &remcomStack[STACKSIZE / sizeof (int) - 1];
 
   exceptionHandler (0, _catchException0);
@@ -954,7 +954,7 @@ set_debug_traps (void)
      "$?#xx") sitting there.  */
   putDebugChar ('+');
 
-  initialized = 1;
+  initialized = true;
 
 }
 
