@@ -1,12 +1,12 @@
 /*
- *  COPYRIGHT (c) 1989-1999.
+ *  COPYRIGHT (c) 1989-2007.
  *  On-Line Applications Research Corporation (OAR).
  *
  *  The license and distribution terms for this file may be
  *  found in the file LICENSE in this distribution or at
  *  http://www.rtems.com/license/LICENSE.
  *
- *  $Id: frsm_tsk.c,v 1.10 2004/04/15 04:00:25 ralf Exp $
+ *  $Id$
  */
 
 #if HAVE_CONFIG_H
@@ -37,7 +37,9 @@ ER frsm_tsk(
 
   the_thread = _ITRON_Task_Get( tskid, &location );
   switch ( location ) {
+#if defined(RTEMS_MULTIPROCESSING)
     case OBJECTS_REMOTE:
+#endif
     case OBJECTS_ERROR:
       return _ITRON_Task_Clarify_get_id_error( tskid );
 
@@ -51,7 +53,7 @@ ER frsm_tsk(
       if ( ! _States_Is_suspended(the_thread->current_state) )
         _ITRON_return_errorno( E_OK );
 
-      _Thread_Resume( the_thread, TRUE );
+      _Thread_Resume( the_thread, true );
       break;
   }
 
