@@ -8,7 +8,7 @@
  *  found in the file LICENSE in this distribution or at
  *  http://www.rtems.com/license/LICENSE.
  *
- *  $Id: lstat.c,v 1.4 2005/01/27 13:32:25 ralf Exp $
+ *  $Id$
  */
 
 #if HAVE_CONFIG_H
@@ -17,6 +17,6 @@
 
 #define _STAT_NAME         lstat
 #define _STAT_R_NAME       _lstat_r
-#define _STAT_FOLLOW_LINKS FALSE
+#define _STAT_FOLLOW_LINKS false
 
 #include "stat.c"
