@@ -6,7 +6,7 @@
  * Copyright (C) 2001 OKTET Ltd., St.-Petersburg, Russia
  * Author: Eugeny S. Mints <Eugeny.Mints@oktet.ru>
  *
- * @(#) $Id: fat.c,v 1.11 2005/01/19 04:54:12 ralf Exp $
+ * @(#) $Id$
  */
 
 #if HAVE_CONFIG_H
@@ -26,6 +26,137 @@
 #include "fat.h"
 #include "fat_fat_operations.h"
 
+int
+fat_buf_access(fat_fs_info_t *fs_info, uint32_t   blk, int op_type,
+               rtems_bdbuf_buffer **buf)
+{
+    rtems_status_code sc = RTEMS_SUCCESSFUL;
+    uint8_t           i;
+    bool              sec_of_fat;
+
+
+    if (fs_info->c.state == FAT_CACHE_EMPTY)
+    {
+        if (op_type == FAT_OP_TYPE_READ)
+            sc = rtems_bdbuf_read(fs_info->vol.dev, blk, &fs_info->c.buf);
+        else
+            sc = rtems_bdbuf_get(fs_info->vol.dev, blk, &fs_info->c.buf);
+        if (sc != RTEMS_SUCCESSFUL)
+            rtems_set_errno_and_return_minus_one(EIO);
+        fs_info->c.blk_num = blk;
+        fs_info->c.modified = 0;
+        fs_info->c.state = FAT_CACHE_ACTUAL;
+    }
+
+    sec_of_fat = ((fs_info->c.blk_num >= fs_info->vol.fat_loc) &&
+                  (fs_info->c.blk_num < fs_info->vol.rdir_loc));
+
+    if (fs_info->c.blk_num != blk)
+    {
+        if (fs_info->c.modified)
+        {
+            if (sec_of_fat && !fs_info->vol.mirror)
+                memcpy(fs_info->sec_buf, fs_info->c.buf->buffer,
+                       fs_info->vol.bps);
+
+            sc = rtems_bdbuf_release_modified(fs_info->c.buf);
+            fs_info->c.state = FAT_CACHE_EMPTY;
+            fs_info->c.modified = 0;
+            if (sc != RTEMS_SUCCESSFUL)
+                rtems_set_errno_and_return_minus_one(EIO);
+
+            if (sec_of_fat && !fs_info->vol.mirror)
+            {
+                rtems_bdbuf_buffer *b;
+
+                for (i = 1; i < fs_info->vol.fats; i++)
+                {
+                    sc = rtems_bdbuf_get(fs_info->vol.dev,
+                                         fs_info->c.blk_num +
+                                         fs_info->vol.fat_length * i,
+                                         &b);
+                    if ( sc != RTEMS_SUCCESSFUL)
+                        rtems_set_errno_and_return_minus_one(ENOMEM);
+                    memcpy(b->buffer, fs_info->sec_buf, fs_info->vol.bps);
+                    sc = rtems_bdbuf_release_modified(b);
+                    if ( sc != RTEMS_SUCCESSFUL)
+                        rtems_set_errno_and_return_minus_one(ENOMEM);
+                }
+            }
+        }
+        else
+        {
+            sc = rtems_bdbuf_release(fs_info->c.buf);
+            fs_info->c.state = FAT_CACHE_EMPTY;
+            if (sc != RTEMS_SUCCESSFUL)
+                rtems_set_errno_and_return_minus_one(EIO);
+
+        }
+        if (op_type == FAT_OP_TYPE_READ)
+            sc = rtems_bdbuf_read(fs_info->vol.dev, blk, &fs_info->c.buf);
+        else
+            sc = rtems_bdbuf_get(fs_info->vol.dev, blk, &fs_info->c.buf);
+        if (sc != RTEMS_SUCCESSFUL)
+            rtems_set_errno_and_return_minus_one(EIO);
+        fs_info->c.blk_num = blk;
+        fs_info->c.state = FAT_CACHE_ACTUAL;
+    }
+    *buf = fs_info->c.buf;
+    return RC_OK;
+}
+
+int
+fat_buf_release(fat_fs_info_t *fs_info)
+{
+    rtems_status_code sc = RTEMS_SUCCESSFUL;
+    uint8_t           i;
+    bool              sec_of_fat;
+
+    if (fs_info->c.state == FAT_CACHE_EMPTY)
+        return RC_OK;
+
+    sec_of_fat = ((fs_info->c.blk_num >= fs_info->vol.fat_loc) &&
+                  (fs_info->c.blk_num < fs_info->vol.rdir_loc));
+
+    if (fs_info->c.modified)
+    {
+        if (sec_of_fat && !fs_info->vol.mirror)
+            memcpy(fs_info->sec_buf, fs_info->c.buf->buffer, fs_info->vol.bps);
+
+        sc = rtems_bdbuf_release_modified(fs_info->c.buf);
+        if (sc != RTEMS_SUCCESSFUL)
+            rtems_set_errno_and_return_minus_one(EIO);
+        fs_info->c.modified = 0;
+
+        if (sec_of_fat && !fs_info->vol.mirror)
+        {
+            rtems_bdbuf_buffer *b;
+
+            for (i = 1; i < fs_info->vol.fats; i++)
+            {
+                sc = rtems_bdbuf_get(fs_info->vol.dev,
+                                     fs_info->c.blk_num +
+                                     fs_info->vol.fat_length * i,
+                                     &b);
+                if ( sc != RTEMS_SUCCESSFUL)
+                    rtems_set_errno_and_return_minus_one(ENOMEM);
+                memcpy(b->buffer, fs_info->sec_buf, fs_info->vol.bps);
+                sc = rtems_bdbuf_release_modified(b);
+                if ( sc != RTEMS_SUCCESSFUL)
+                    rtems_set_errno_and_return_minus_one(ENOMEM);
+            }
+        }
+    }
+    else
+    {
+        sc = rtems_bdbuf_release(fs_info->c.buf);
+        if (sc != RTEMS_SUCCESSFUL)
+            rtems_set_errno_and_return_minus_one(EIO);
+    }
+    fs_info->c.state = FAT_CACHE_EMPTY;
+    return RC_OK;
+}
+
 /* _fat_block_read --
  *     This function reads 'count' bytes from device filesystem is mounted on,
  *     starts at 'start+offset' position where 'start' computed in sectors
@@ -57,7 +188,7 @@ _fat_block_read(
     ssize_t                 cmpltd = 0;
     uint32_t                blk = start;
     uint32_t                ofs = offset;
-    bdbuf_buffer           *block = NULL;
+    rtems_bdbuf_buffer     *block = NULL;
     uint32_t                c = 0;
 
     while (count > 0)
@@ -102,13 +233,13 @@ _fat_block_write(
     uint32_t                              count,
     const void                           *buff)
 {
-    int            rc = RC_OK;
-    fat_fs_info_t *fs_info = mt_entry->fs_info;
-    ssize_t        cmpltd = 0;
-    uint32_t       blk  = start;
-    uint32_t       ofs = offset;
-    bdbuf_buffer  *block = NULL;
-    uint32_t       c = 0;
+    int                 rc = RC_OK;
+    fat_fs_info_t      *fs_info = mt_entry->fs_info;
+    ssize_t             cmpltd = 0;
+    uint32_t            blk  = start;
+    uint32_t            ofs = offset;
+    rtems_bdbuf_buffer *block = NULL;
+    uint32_t            c = 0;
 
     while(count > 0)
     {
@@ -133,8 +264,23 @@ _fat_block_write(
     return cmpltd;
 }
 
-
-
+/* _fat_block_release --
+ *     This function works around the hack that hold a bdbuf and does
+ *     not release it.
+ *
+ * PARAMETERS:
+ *     mt_entry - mount table entry
+ *
+ * RETURNS:
+ *     0 on success, or -1 if error occured and errno set appropriately
+ */
+int
+_fat_block_release(
+    rtems_filesystem_mount_table_entry_t *mt_entry)
+{
+    fat_fs_info_t *fs_info = mt_entry->fs_info;
+    return fat_buf_release(fs_info);
+}
 
 /* fat_cluster_read --
  *     wrapper for reading a whole cluster at once
@@ -205,6 +351,7 @@ fat_cluster_write(
 int
 fat_init_volume_info(rtems_filesystem_mount_table_entry_t *mt_entry)
 {
+    rtems_status_code   sc = RTEMS_SUCCESSFUL;
     int                 rc = RC_OK;
     fat_fs_info_t      *fs_info = mt_entry->fs_info;
     register fat_vol_t *vol = &fs_info->vol;
@@ -212,50 +359,53 @@ fat_init_volume_info(rtems_filesystem_mount_table_entry_t *mt_entry)
     char                boot_rec[FAT_MAX_BPB_SIZE];
     char                fs_info_sector[FAT_USEFUL_INFO_SIZE];
     ssize_t             ret = 0;
-    int                 fd;
     struct stat         stat_buf;
     int                 i = 0;
+    rtems_bdbuf_buffer *block = NULL;
 
     rc = stat(mt_entry->dev, &stat_buf);
     if (rc == -1)
         return rc;
 
-    /* rtmes feature: no block devices, all are character devices */
-    if (!S_ISCHR(stat_buf.st_mode))
-        set_errno_and_return_minus_one(ENOTBLK);
+    /* Must be a block device. */
+    if (!S_ISBLK(stat_buf.st_mode))
+        rtems_set_errno_and_return_minus_one(ENOTTY);
 
-    /* check that  device is registred as block device and lock it */
-    vol->dd = rtems_disk_lookup(stat_buf.st_dev);
+    /* check that device is registred as block device and lock it */
+    vol->dd = rtems_disk_obtain(stat_buf.st_rdev);
     if (vol->dd == NULL)
-        set_errno_and_return_minus_one(ENOTBLK);
+        rtems_set_errno_and_return_minus_one(EIO);
 
-    vol->dev = stat_buf.st_dev;
+    vol->dev = stat_buf.st_rdev;
 
-    fd = open(mt_entry->dev, O_RDONLY);
-    if (fd == -1)
+    /* Read boot record */
+    /* FIXME: Asserts FAT_MAX_BPB_SIZE < bdbuf block size */
+    sc = rtems_bdbuf_read( vol->dev, 0, &block);
+    if (sc != RTEMS_SUCCESSFUL)
     {
         rtems_disk_release(vol->dd);
-        return -1;
+        rtems_set_errno_and_return_minus_one( EIO);
     }
 
-    ret = read(fd, (void *)boot_rec, FAT_MAX_BPB_SIZE);
-    if ( ret != FAT_MAX_BPB_SIZE )
+    memcpy( boot_rec, block->buffer, FAT_MAX_BPB_SIZE);
+
+    sc = rtems_bdbuf_release( block);
+    if (sc != RTEMS_SUCCESSFUL)
     {
-        close(fd);
         rtems_disk_release(vol->dd);
-        set_errno_and_return_minus_one( EIO );
+        rtems_set_errno_and_return_minus_one( EIO );
     }
-    close(fd);
 
+    /* Evaluate boot record */
     vol->bps = FAT_GET_BR_BYTES_PER_SECTOR(boot_rec);
- 
-    if ( (vol->bps != 512)  && 
-         (vol->bps != 1024) && 
+
+    if ( (vol->bps != 512)  &&
+         (vol->bps != 1024) &&
          (vol->bps != 2048) &&
          (vol->bps != 4096))
     {
         rtems_disk_release(vol->dd);
-        set_errno_and_return_minus_one( EINVAL );
+        rtems_set_errno_and_return_minus_one( EINVAL );
     }
 
     for (vol->sec_mul = 0, i = (vol->bps >> FAT_SECTOR512_BITS); (i & 1) == 0;
@@ -271,7 +421,7 @@ fat_init_volume_info(rtems_filesystem_mount_table_entry_t *mt_entry)
     if (vol->spc == 0)
     {
         rtems_disk_release(vol->dd);
-        set_errno_and_return_minus_one(EINVAL);
+        rtems_set_errno_and_return_minus_one(EINVAL);
     }
 
     for (vol->spc_log2 = 0, i = vol->spc; (i & 1) == 0;
@@ -283,7 +433,7 @@ fat_init_volume_info(rtems_filesystem_mount_table_entry_t *mt_entry)
     if ((vol->bpc = vol->bps << vol->spc_log2) > MS_BYTES_PER_CLUSTER_LIMIT)
     {
         rtems_disk_release(vol->dd);
-        set_errno_and_return_minus_one(EINVAL);
+        rtems_set_errno_and_return_minus_one(EINVAL);
     }
 
     for (vol->bpc_log2 = 0, i = vol->bpc; (i & 1) == 0;
@@ -293,7 +443,7 @@ fat_init_volume_info(rtems_filesystem_mount_table_entry_t *mt_entry)
     vol->fat_loc = FAT_GET_BR_RESERVED_SECTORS_NUM(boot_rec);
 
     vol->rdir_entrs = FAT_GET_BR_FILES_PER_ROOT_DIR(boot_rec);
-    
+
     /* calculate the count of sectors occupied by the root directory */
     vol->rdir_secs = ((vol->rdir_entrs * FAT_DIRENTRY_SIZE) + (vol->bps - 1)) /
                      vol->bps;
@@ -304,18 +454,18 @@ fat_init_volume_info(rtems_filesystem_mount_table_entry_t *mt_entry)
         vol->fat_length = FAT_GET_BR_SECTORS_PER_FAT(boot_rec);
     else
         vol->fat_length = FAT_GET_BR_SECTORS_PER_FAT32(boot_rec);
-  
-    vol->data_fsec = vol->fat_loc + vol->fats * vol->fat_length + 
+
+    vol->data_fsec = vol->fat_loc + vol->fats * vol->fat_length +
                      vol->rdir_secs;
 
     /* for  FAT12/16 root dir starts at(sector) */
     vol->rdir_loc = vol->fat_loc + vol->fats * vol->fat_length;
-  
+
     if ( (FAT_GET_BR_TOTAL_SECTORS_NUM16(boot_rec)) != 0)
         vol->tot_secs = FAT_GET_BR_TOTAL_SECTORS_NUM16(boot_rec);
     else
         vol->tot_secs = FAT_GET_BR_TOTAL_SECTORS_NUM32(boot_rec);
-  
+
     data_secs = vol->tot_secs - vol->data_fsec;
 
     vol->data_cls = data_secs / vol->spc;
@@ -346,7 +496,7 @@ fat_init_volume_info(rtems_filesystem_mount_table_entry_t *mt_entry)
     if (vol->type == FAT_FAT32)
     {
         vol->rdir_cl = FAT_GET_BR_FAT32_ROOT_CLUSTER(boot_rec);
-      
+
         vol->mirror = FAT_GET_BR_EXT_FLAGS(boot_rec) & FAT_BR_EXT_FLAGS_MIRROR;
         if (vol->mirror)
             vol->afat = FAT_GET_BR_EXT_FLAGS(boot_rec) & FAT_BR_EXT_FLAGS_FAT_NUM;
@@ -357,7 +507,7 @@ fat_init_volume_info(rtems_filesystem_mount_table_entry_t *mt_entry)
         if( vol->info_sec == 0 )
         {
             rtems_disk_release(vol->dd);
-            set_errno_and_return_minus_one( EINVAL );
+            rtems_set_errno_and_return_minus_one( EINVAL );
         }
         else
         {
@@ -367,13 +517,14 @@ fat_init_volume_info(rtems_filesystem_mount_table_entry_t *mt_entry)
             {
                 rtems_disk_release(vol->dd);
                 return -1;
-            }    
-      
-            if (FAT_GET_FSINFO_LEAD_SIGNATURE(fs_info_sector) != 
+            }
+
+            if (FAT_GET_FSINFO_LEAD_SIGNATURE(fs_info_sector) !=
                 FAT_FSINFO_LEAD_SIGNATURE_VALUE)
             {
+                _fat_block_release(mt_entry);
                 rtems_disk_release(vol->dd);
-                set_errno_and_return_minus_one( EINVAL );
+                rtems_set_errno_and_return_minus_one( EINVAL );
             }
             else
             {
@@ -381,16 +532,18 @@ fat_init_volume_info(rtems_filesystem_mount_table_entry_t *mt_entry)
                                       FAT_USEFUL_INFO_SIZE, fs_info_sector);
                 if ( ret < 0 )
                 {
+                    _fat_block_release(mt_entry);
                     rtems_disk_release(vol->dd);
                     return -1;
-                }    
-                    
+                }
+
                 vol->free_cls = FAT_GET_FSINFO_FREE_CLUSTER_COUNT(fs_info_sector);
                 vol->next_cl = FAT_GET_FSINFO_NEXT_FREE_CLUSTER(fs_info_sector);
-                rc = fat_fat32_update_fsinfo_sector(mt_entry, 0xFFFFFFFF, 
+                rc = fat_fat32_update_fsinfo_sector(mt_entry, 0xFFFFFFFF,
                                                     0xFFFFFFFF);
                 if ( rc != RC_OK )
                 {
+                    _fat_block_release(mt_entry);
                     rtems_disk_release(vol->dd);
                     return rc;
                 }
@@ -405,28 +558,31 @@ fat_init_volume_info(rtems_filesystem_mount_table_entry_t *mt_entry)
         vol->free_cls = 0xFFFFFFFF;
         vol->next_cl = 0xFFFFFFFF;
     }
+
+    _fat_block_release(mt_entry);
+
     vol->afat_loc = vol->fat_loc + vol->fat_length * vol->afat;
 
     /* set up collection of fat-files fd */
-    fs_info->vhash = calloc(FAT_HASH_SIZE, sizeof(Chain_Control));
+    fs_info->vhash = calloc(FAT_HASH_SIZE, sizeof(rtems_chain_control));
     if ( fs_info->vhash == NULL )
     {
         rtems_disk_release(vol->dd);
-        set_errno_and_return_minus_one( ENOMEM );
+        rtems_set_errno_and_return_minus_one( ENOMEM );
     }
 
     for (i = 0; i < FAT_HASH_SIZE; i++)
-        _Chain_Initialize_empty(fs_info->vhash + i);
+        rtems_chain_initialize_empty(fs_info->vhash + i);
 
-    fs_info->rhash = calloc(FAT_HASH_SIZE, sizeof(Chain_Control));
+    fs_info->rhash = calloc(FAT_HASH_SIZE, sizeof(rtems_chain_control));
     if ( fs_info->rhash == NULL )
     {
         rtems_disk_release(vol->dd);
         free(fs_info->vhash);
-        set_errno_and_return_minus_one( ENOMEM );
+        rtems_set_errno_and_return_minus_one( ENOMEM );
     }
     for (i = 0; i < FAT_HASH_SIZE; i++)
-        _Chain_Initialize_empty(fs_info->rhash + i);
+        rtems_chain_initialize_empty(fs_info->rhash + i);
 
     fs_info->uino_pool_size = FAT_UINO_POOL_INIT_SIZE;
     fs_info->uino_base = (vol->tot_secs << vol->sec_mul) << 4;
@@ -437,7 +593,7 @@ fat_init_volume_info(rtems_filesystem_mount_table_entry_t *mt_entry)
         rtems_disk_release(vol->dd);
         free(fs_info->vhash);
         free(fs_info->rhash);
-        set_errno_and_return_minus_one( ENOMEM );
+        rtems_set_errno_and_return_minus_one( ENOMEM );
     }
     fs_info->sec_buf = (uint8_t *)calloc(vol->bps, sizeof(uint8_t));
     if (fs_info->sec_buf == NULL)
@@ -446,7 +602,7 @@ fat_init_volume_info(rtems_filesystem_mount_table_entry_t *mt_entry)
         free(fs_info->vhash);
         free(fs_info->rhash);
         free(fs_info->uino);
-        set_errno_and_return_minus_one( ENOMEM );
+        rtems_set_errno_and_return_minus_one( ENOMEM );
     }
 
     return RC_OK;
@@ -484,19 +640,19 @@ fat_shutdown_drive(rtems_filesystem_mount_table_entry_t *mt_entry)
 
     for (i = 0; i < FAT_HASH_SIZE; i++)
     {
-        Chain_Node    *node = NULL;
-        Chain_Control *the_chain = fs_info->vhash + i;
+        rtems_chain_node    *node = NULL;
+        rtems_chain_control *the_chain = fs_info->vhash + i;
 
-        while ( (node = _Chain_Get(the_chain)) != NULL )
+        while ( (node = rtems_chain_get(the_chain)) != NULL )
             free(node);
     }
 
     for (i = 0; i < FAT_HASH_SIZE; i++)
     {
-        Chain_Node    *node = NULL;
-        Chain_Control *the_chain = fs_info->rhash + i;
+        rtems_chain_node    *node = NULL;
+        rtems_chain_control *the_chain = fs_info->rhash + i;
 
-        while ( (node = _Chain_Get(the_chain)) != NULL )
+        while ( (node = rtems_chain_get(the_chain)) != NULL )
             free(node);
     }
 
@@ -537,7 +693,7 @@ fat_init_clusters_chain(
 
     buf = calloc(fs_info->vol.bpc, sizeof(char));
     if ( buf == NULL )
-        set_errno_and_return_minus_one( EIO );
+        rtems_set_errno_and_return_minus_one( EIO );
 
     while ((cur_cln & fs_info->vol.mask) < fs_info->vol.eoc_val)
     {
@@ -590,7 +746,7 @@ fat_get_unique_ino(rtems_filesystem_mount_table_entry_t *mt_entry)
 {
     register fat_fs_info_t *fs_info = mt_entry->fs_info;
     uint32_t                j = 0;
-    rtems_boolean           resrc_unsuff = FALSE;
+    bool                    resrc_unsuff = false;
 
     while (!resrc_unsuff)
     {
@@ -613,10 +769,10 @@ fat_get_unique_ino(rtems_filesystem_mount_table_entry_t *mt_entry)
             if (fs_info->uino != NULL)
                 fs_info->index = fs_info->uino_pool_size;
             else
-                resrc_unsuff = TRUE;
+                resrc_unsuff = true;
         }
         else
-            resrc_unsuff = TRUE;
+            resrc_unsuff = true;
     }
     return 0;
 }
@@ -650,9 +806,9 @@ fat_free_unique_ino(
  *     ino   - ino to be tested
  *
  * RETURNS:
- *     TRUE if ino is allocated from unique ino pool, FALSE otherwise
+ *     true if ino is allocated from unique ino pool, false otherwise
  */
-inline rtems_boolean
+inline bool
 fat_ino_is_unique(
     rtems_filesystem_mount_table_entry_t *mt_entry,
     uint32_t                              ino
