@@ -1,5 +1,5 @@
 /*	
- *  DCHP client for RTEMS 
+ *  DCHP client for RTEMS
  *  Andrew Bythell, <abythell@nortelnetworks.com>
  *  based on and uses subroutines from c/src/libnetworking/nfs/bootp_subr.c
  */
@@ -9,6 +9,12 @@
  * Brendan Gannon, <bgannon@cybertec.com.au>
  */
 
+/*
+ * Added interface to terminate DHCP task, and removed panics.
+ * Arnout Vandecappelle <arnout@mind.be>, Essensium/Mind
+ * Maarten Van Es <maarten@mind.be>, Essensium/Mind
+ */
+
 /*
  * Copyright (c) 1995 Gordon Ross, Adam Glass
  * Copyright (c) 1992 Regents of the University of California.
@@ -55,14 +61,6 @@
  *   the end of the file should be removed
  */
 
-#ifndef _COMPILING_BSD_KERNEL_
-#define _COMPILING_BSD_KERNEL_
-#endif
-
-#ifndef KERNEL
-#define KERNEL
-#endif
-
 #ifndef __INSIDE_RTEMS_BSD_TCPIP_STACK__
 #define __INSIDE_RTEMS_BSD_TCPIP_STACK__
 #endif
@@ -168,7 +166,7 @@ struct dhcp_packet
   unsigned char  vend[312];
 };
 
-/* 
+/*
  * External Declarations for Functions found in
  * rtems/c/src/libnetworking/nfs/
  */
@@ -186,7 +184,6 @@ extern int bootpc_adjust_interface (struct ifreq *ireq,
                                     struct proc *procp);
 extern void *bootp_strdup_realloc (char *dst,
                                    const char *src);
-extern int nfs_diskless_valid;
 
 /*
  * Variables
@@ -239,11 +236,11 @@ printsetup (const char     *iface,
   char ip_str[15];
 
   printf ("dhcpc: %s: ", iface);
-  
+
   ip = ntohl (ip_addr.s_addr);
   format_ip (ip, ip_str);
   printf ("inet: %-15s ", ip_str);
-  
+
   ip = ntohl (mask_addr.s_addr);
   format_ip (ip, ip_str);
   printf ("mask: %-15s\n", ip_str);
@@ -251,11 +248,11 @@ printsetup (const char     *iface,
   ip = ntohl (srv_addr.s_addr);
   format_ip (ip, ip_str);
   printf   ("             srv: %-15s ", ip_str);
-  
+
   ip = ntohl (gw_addr.s_addr);
   format_ip (ip, ip_str);
   printf ("  gw: %-15s\n", ip_str);
-  
+
   return;
 }
 
@@ -271,6 +268,8 @@ process_options (unsigned char *optbuf, int optbufSize)
   int code, ncode;
   char *p;
 
+  dhcp_message_type = 0;
+
   ncode = optbuf[0];
   while (j < optbufSize)
   {
@@ -284,13 +283,13 @@ process_options (unsigned char *optbuf, int optbufSize)
     }
     len = optbuf[j + 1];
     j += 2;
-    
+
     if ((len + j) >= optbufSize)
     {
       printf ("Truncated field for code %d", code);
       return;
     }
-    
+
     ncode = optbuf[j + len];
     optbuf[j + len] = '\0';
     p = (char*) &optbuf[j];
@@ -303,24 +302,30 @@ process_options (unsigned char *optbuf, int optbufSize)
     {
       case 1:
         /* Subnet mask */
-        if (len != 4)
-          panic ("dhcpc: subnet mask len is %d", len);
+        if (len != 4) {
+          printf ("dhcpc: subnet mask len is %d\n", len);
+          continue;
+        }
         memcpy (&dhcp_netmask.sin_addr, p, 4);
         dhcp_gotnetmask = 1;
         break;
 
       case 2:
         /* Time offset */
-        if (len != 4)
-          panic ("dhcpc: time offset len is %d", len);
+        if (len != 4) {
+          printf ("dhcpc: time offset len is %d\n", len);
+          continue;
+        }
         memcpy (&rtems_bsdnet_timeoffset, p, 4);
         rtems_bsdnet_timeoffset = ntohl (rtems_bsdnet_timeoffset);
         break;
 
       case 3:
         /* Routers */
-        if (len % 4)
-          panic ("dhcpc: Router Len is %d", len);
+        if (len % 4) {
+          printf ("dhcpc: Router Len is %d\n", len);
+          continue;
+        }
         if (len > 0)
         {
           memcpy (&dhcp_gw.sin_addr, p, 4);
@@ -330,8 +335,10 @@ process_options (unsigned char *optbuf, int optbufSize)
 
       case 42:
         /* NTP servers */
-        if (len % 4)
-          panic ("dhcpc: time server Len is %d", len);
+        if (len % 4) {
+          printf ("dhcpc: time server Len is %d\n", len);
+          continue;
+        }
         {
           int tlen = 0;
           while ((tlen < len) &&
@@ -349,8 +356,10 @@ process_options (unsigned char *optbuf, int optbufSize)
 
       case 6:
         /* Domain Name servers */
-        if (len % 4)
-          panic ("dhcpc: DNS Len is %d", len);
+        if (len % 4) {
+          printf ("dhcpc: DNS Len is %d\n", len);
+          continue;
+        }
         {
           int dlen = 0;
           while ((dlen < len) &&
@@ -368,10 +377,12 @@ process_options (unsigned char *optbuf, int optbufSize)
 
       case 12:
         /* Host name */
-        if (len >= MAXHOSTNAMELEN)
-          panic ("dhcpc: hostname >= %d bytes", MAXHOSTNAMELEN);
+        if (len >= MAXHOSTNAMELEN) {
+          printf ("dhcpc: hostname >= %d bytes\n", MAXHOSTNAMELEN);
+          len = MAXHOSTNAMELEN-1;
+        }
         if (sethostname (p, len) < 0)
-          panic ("dhcpc: can't set host name");
+          printf ("dhcpc: can't set host name");
         if (dhcp_hostname != NULL)
         {
           dhcp_hostname = realloc (dhcp_hostname, len);
@@ -383,8 +394,10 @@ process_options (unsigned char *optbuf, int optbufSize)
 
       case 7:
         /* Log servers */
-        if (len % 4)
-          panic ("dhcpc: Log server Len is %d", len);
+        if (len % 4) {
+          printf ("dhcpc: Log server Len is %d\n", len);
+          continue;
+        }
         if (len > 0)
         {
           memcpy (&rtems_bsdnet_log_host_address, p, 4);
@@ -406,8 +419,8 @@ process_options (unsigned char *optbuf, int optbufSize)
       case 50:
         /* DHCP Requested IP Address */
         if (len != 4)
-          panic ("dhcpc: DHCP option requested IP len is %d", len);
-        /* 
+          printf ("dhcpc: DHCP option requested IP len is %d", len);
+        /*
          * although nothing happens here, this case keeps the client
          * from complaining about unknown options.  The Requested IP
          * is necessary to return to the server for a DHCP REQUEST
@@ -416,27 +429,33 @@ process_options (unsigned char *optbuf, int optbufSize)
 
       case 51:
         /* DHCP Lease Length */
-        if (len != 4)
-          panic ("dhcpc: DHCP option lease-length len is %d", len);
+        if (len != 4) {
+          printf ("dhcpc: DHCP option lease-length len is %d", len);
+          continue;
+        }
         memcpy (&dhcp_lease_time, &p[0], 4);
         dhcp_lease_time = ntohl (dhcp_lease_time);
         break;
-        
+
       case 52:
         /* DHCP option override */
-        if (len != 1)
-          panic ("dhcpc: DHCP option overload len is %d", len);
+        if (len != 1) {
+          printf ("dhcpc: DHCP option overload len is %d", len);
+          continue;
+        }
         dhcp_option_overload = p[0];
         break;
 
       case 53:
         /* DHCP message */
-        if (len != 1)
-          panic ("dhcpc: DHCP message len is %d", len);
+        if (len != 1) {
+          printf ("dhcpc: DHCP message len is %d", len);
+          continue;
+        }
         dhcp_message_type = p[0];
         break;
 
-      case 128:		/* Site-specific option for DHCP servers that 
+      case 128:		/* Site-specific option for DHCP servers that
                    *   a) don't supply tag 54
                    * and
                    *   b) don't supply the server address in siaddr
@@ -447,8 +466,10 @@ process_options (unsigned char *optbuf, int optbufSize)
                    */
       case 54:
         /* DHCP server */
-        if (len != 4)
-          panic ("dhcpc: DHCP server len is %d", len);
+        if (len != 4) {
+          printf ("dhcpc: DHCP server len is %d", len);
+          continue;
+        }
         memcpy (&rtems_bsdnet_bootp_server_address, p, 4);
         dhcp_gotserver = 1;
         break;
@@ -480,7 +501,7 @@ dhcp_discover_req (struct dhcp_packet* call,
                    unsigned long *xid)
 {
   int len = 0;
-  
+
   memset (call, 0, sizeof (struct dhcp_packet));
 
   /*
@@ -493,7 +514,7 @@ dhcp_discover_req (struct dhcp_packet* call,
   (*xid)++;
   call->xid = htonl (*xid);
   call->flags = htons (DHCP_BROADCAST);
-  
+
   memcpy (&call->chaddr, LLADDR (sdl), sdl->sdl_alen);
 
   /*
@@ -546,7 +567,7 @@ dhcp_request_req (struct dhcp_packet* call,
   int           len = 0;
   unsigned long temp;
   char          *hostname;
-  
+
   memset (call, 0, sizeof (struct dhcp_packet));
 
   /*
@@ -598,7 +619,7 @@ dhcp_request_req (struct dhcp_packet* call,
   call->vend[len++] = sizeof (reply->yiaddr);
   memcpy (&call->vend[len], &reply->yiaddr, sizeof (reply->yiaddr));
   len += sizeof (reply->yiaddr);
-  
+
   /*
    * DHCP Parameter request list
    */
@@ -667,27 +688,51 @@ dhcp_task (rtems_task_argument _sdl)
   unsigned int        timeout = 0;
   int                 error;
   struct proc *procp = NULL;
-  
+  int                 disconnected;
+  rtems_status_code   ev_st;
+
   sdl = (struct sockaddr_dl *) _sdl;
-  
+
   count = dhcp_elapsed_time;
-  
-  while (TRUE)
+  disconnected = 0;
+
+  while (true)
   {
+    /*
+     * Sleep until the next poll
+     */
+    timeout = TOD_MILLISECONDS_TO_TICKS (1000);
+    ev_st = rtems_event_receive (RTEMS_EVENT_0,
+                                 RTEMS_WAIT | RTEMS_EVENT_ANY,
+                                 timeout, &event_out);
+
+    /*
+     * Check if not a poll timeout. So when ANY event received, exit task.
+     * Actually, only event RTEMS_EVENT_0 sent from rtem_dhcp_failsafe.c
+     * if "failsafe" dhcp enabled when interface down.  Otherwise, no
+     * event should occur, just timeout.
+     */
+    if(ev_st != RTEMS_TIMEOUT)
+        break;
+
+    count++;
+
     if (count >= (dhcp_lease_time / 2))
     {
       rtems_bsdnet_semaphore_obtain ();
-      
-      dhcp_request_req (&call, &dhcp_req, sdl, TRUE);
+
+      dhcp_request_req (&call, &dhcp_req, sdl, true);
 
       /*
        * Send the Request.
        */
       error = bootpc_call (&call, &dhcp_req, procp);
-      
-      if (error)
-        panic ("DHCP call failed -- error %d", error);
-      
+      if (error) {
+        rtems_bsdnet_semaphore_release ();
+        printf ("DHCP call failed -- error %d", error);
+        continue;
+      }
+
       /*
        * Check for DHCP ACK/NACK
        */
@@ -696,31 +741,30 @@ dhcp_task (rtems_task_argument _sdl)
                   sizeof (dhcp_magic_cookie)) != 0)
       {
         rtems_bsdnet_semaphore_release ();
-        panic ("DHCP server did not send Magic Cookie.\n");
+        printf ("DHCP server did not send Magic Cookie.\n");
+        continue;
       }
-  
+
       process_options (&dhcp_req.vend[4], sizeof (dhcp_req.vend) - 4);
-  
+
       if (dhcp_message_type != DHCP_ACK)
       {
         rtems_bsdnet_semaphore_release ();
-        panic ("DHCP server did not accept the DHCP request");
+        printf ("DHCP server did not accept the DHCP request");
+        continue;
       }
-      
+
       rtems_bsdnet_semaphore_release ();
-      
+
       count = 0;
     }
-
-    /*
-     * Sleep until the next poll
-     */
-    timeout = TOD_MILLISECONDS_TO_TICKS (1000);
-    rtems_event_receive (RTEMS_EVENT_0,
-                         RTEMS_WAIT | RTEMS_EVENT_ANY,
-                         timeout, &event_out);
-    count++;
   }
+
+
+  dhcp_task_id = 0;
+  printf ("dhcpc: exiting lease renewal task.\n");
+  rtems_task_delete( RTEMS_SELF);
+
 }
 
 /*
@@ -732,7 +776,7 @@ dhcp_start_task (struct sockaddr_dl *sdl,
                  int priority)
 {
   rtems_status_code sc;
-  
+
   memcpy (&dhcp_req, reply, sizeof (struct dhcp_packet));
 
   sc = rtems_task_create (rtems_build_name ('d','h','c','p'),
@@ -766,14 +810,14 @@ dhcp_interface_has_ip (struct ifreq *ireq, struct socket *so, struct proc *procp
 {
   struct sockaddr_in* sin;
   int error;
-  
+
   /*
    * Check if the interface is already up.
    */
   error = ifioctl(so, SIOCGIFFLAGS, (caddr_t)ireq, procp);
   if (error)
     return 0;
-  
+
   if ((ireq->ifr_flags & IFF_UP) == 0)
     return 0;
 
@@ -792,11 +836,15 @@ dhcp_interface_has_ip (struct ifreq *ireq, struct socket *so, struct proc *procp
 }
 
 /*
- *  DCHP Client Routine 
+ *  DCHP Client Routine
  *    - The first DHCP offer is always accepted
  *    - No DHCP DECLINE message is sent if ARPing fails
+ *
+ *    return value:
+ *    0: ok
+ *    < 0: failed to startup or configure interface
  */
-void
+int
 dhcp_init (int update_files)
 {
   struct dhcp_packet   call;
@@ -811,12 +859,6 @@ dhcp_init (int update_files)
   struct ifaddr        *ifa;
   struct sockaddr_dl   *sdl = NULL;
   struct proc          *procp = NULL;
-  
-  /*
-   * If already filled in, don't touch it here 
-   */
-  if (nfs_diskless_valid)
-    return;
 
   /*
    * If we are to update the files create the root
@@ -827,20 +869,25 @@ dhcp_init (int update_files)
       printf("Error creating the root filesystem.\nFile not created.\n");
       update_files = 0;
     }
-  
+
   /*
    * Find a network interface.
    */
   for (ifp = ifnet; ifp != 0; ifp = ifp->if_next)
     if ((ifp->if_flags & (IFF_LOOPBACK | IFF_POINTOPOINT)) == 0)
       break;
-  if (ifp == NULL)
-    panic ("dhcpc_init: no suitable interface");
+  if (ifp == NULL){
+    printf ("dhcpc_init: no suitable interface\n");
+    return -1;
+  }
+
   memset (&ireq, 0, sizeof (ireq));
   sprintf (ireq.ifr_name, "%s%d", ifp->if_name, ifp->if_unit);
 
-  if ((error = socreate (AF_INET, &so, SOCK_DGRAM, 0, procp)) != 0)
-    panic ("nfs_boot: socreate, error=%d", error);
+  if ((error = socreate (AF_INET, &so, SOCK_DGRAM, 0, procp)) != 0) {
+    printf ("dhcpc_init: socreate, error: %s\n", strerror(error));
+    return -1;
+  }
 
   if (!dhcp_interface_has_ip (&ireq, so, procp))
     bootpc_fakeup_interface (&ireq, so, procp);
@@ -854,11 +901,17 @@ dhcp_init (int update_files)
         sdl->sdl_type == IFT_ETHER)
       break;
 
-  if (!sdl)
-    panic ("dhcpc: Unable to find HW address");
-  if (sdl->sdl_alen != EALEN)
-    panic ("dhcpc: HW address len is %d, expected value is %d",
+  if (!sdl){
+    printf ("dhcpc_init: Unable to find HW address\n");
+    soclose (so);
+    return -1;
+  }
+  if (sdl->sdl_alen != EALEN) {
+    printf ("dhcpc_init: HW address len is %d, expected value is %d\n",
            sdl->sdl_alen, EALEN);
+    soclose (so);
+    return -1;
+  }
 
   /*
    * Build the DHCP Discover
@@ -869,39 +922,57 @@ dhcp_init (int update_files)
    * Send the Discover.
    */
   error = bootpc_call (&call, &reply, procp);
-  if (error)
-    panic ("BOOTP call failed -- error %d", error);
+  if (error) {
+    printf ("BOOTP call failed -- %s\n", strerror(error));
+    soclose (so);
+    return -1;
+  }
 
   /*
    * Check for DHCP OFFER
    */
-  if (memcmp (&reply.vend[0], dhcp_magic_cookie, sizeof (dhcp_magic_cookie)) != 0)
-    panic ("DHCP server did not send Magic Cookie.\n");
+  if (memcmp (&reply.vend[0], dhcp_magic_cookie, sizeof (dhcp_magic_cookie)) != 0) {
+    printf ("DHCP server did not send Magic Cookie.\n");
+    soclose (so);
+    return -1;
+  }
 
   process_options (&reply.vend[4], sizeof (reply.vend) - 4);
-  
-  if (dhcp_message_type != DHCP_OFFER)
-    panic ("DHCP server did not send a DHCP Offer.\n");
+
+  if (dhcp_message_type != DHCP_OFFER) {
+    printf ("DHCP server did not send a DHCP Offer.\n");
+    soclose (so);
+    return -1;
+  }
 
   /*
-   * Send a DHCP REQUEST 
+   * Send a DHCP REQUEST
    */
-  dhcp_request_req (&call, &reply, sdl, TRUE);
-  
+  dhcp_request_req (&call, &reply, sdl, true);
+
   error = bootpc_call (&call, &reply, procp);
-  if (error)
-    panic ("BOOTP call failed -- error %d", error);
+  if (error) {
+    printf ("BOOTP call failed -- %s\n", strerror(error));
+    soclose (so);
+    return -1;
+  }
 
   /*
    * Check for DHCP ACK/NACK
    */
-  if (memcmp (&reply.vend[0], dhcp_magic_cookie, sizeof (dhcp_magic_cookie)) != 0)
-    panic ("DHCP server did not send Magic Cookie.\n");
-  
+  if (memcmp (&reply.vend[0], dhcp_magic_cookie, sizeof (dhcp_magic_cookie)) != 0) {
+    printf ("DHCP server did not send Magic Cookie.\n");
+    soclose (so);
+    return -1;
+  }
+
   process_options (&reply.vend[4], sizeof (reply.vend) - 4);
-  
-  if (dhcp_message_type != DHCP_ACK)
-    panic ("DHCP server did not accept the DHCP request");
+
+  if (dhcp_message_type != DHCP_ACK) {
+    printf ("DHCP server did not accept the DHCP request\n");
+    soclose (so);
+    return -1;
+  }
 
   /*
    * Initialize network address structures
@@ -926,13 +997,13 @@ dhcp_init (int update_files)
    */
   if (memcmp (&reply.vend[0], dhcp_magic_cookie, sizeof (dhcp_magic_cookie)) == 0)
     process_options (&reply.vend[4], sizeof (reply.vend) - 4);
-  
+
   if (dhcp_option_overload & 1)
     process_options ((unsigned char*) reply.file, sizeof reply.file);
   else
     if (reply.file[0])
       rtems_bsdnet_bootp_boot_file_name = strdup (reply.file);
-  
+
   if (dhcp_option_overload & 2)
     process_options ((unsigned char*) reply.sname, sizeof reply.sname);
   else
@@ -951,13 +1022,13 @@ dhcp_init (int update_files)
     else
       dhcp_netmask.sin_addr.s_addr = htonl (IN_CLASSC_NET);
   }
-  
+
   if (!dhcp_gotserver)
     rtems_bsdnet_bootp_server_address = reply.siaddr;
-  
+
   if (!dhcp_gotgw)
     dhcp_gw.sin_addr = reply.giaddr;
-  
+
   if (!dhcp_gotlogserver)
     rtems_bsdnet_log_host_address = rtems_bsdnet_bootp_server_address;
 
@@ -977,7 +1048,7 @@ dhcp_init (int update_files)
       hn = "me";
       sethostname (hn, strlen (hn));
     }
-    rtems_rootfs_append_host_rec(*((unsigned long*) &myaddr.sin_addr), hn, dn);
+    rtems_rootfs_append_host_rec(myaddr.sin_addr.s_addr, hn, dn);
 
     /*
      * Should the given domainname be used here ?
@@ -987,12 +1058,12 @@ dhcp_init (int update_files)
         hn = rtems_bsdnet_bootp_server_name;
       else
         hn = "bootps";
-      rtems_rootfs_append_host_rec(*((unsigned long *) &rtems_bsdnet_bootp_server_address),
+      rtems_rootfs_append_host_rec(rtems_bsdnet_bootp_server_address.s_addr,
                                    hn, dn);
     }
 
     if (dhcp_gotlogserver) {
-      rtems_rootfs_append_host_rec(*((unsigned long *) &rtems_bsdnet_log_host_address),
+      rtems_rootfs_append_host_rec(rtems_bsdnet_log_host_address.s_addr,
                                    "logs", dn);
     }
 
@@ -1005,9 +1076,9 @@ dhcp_init (int update_files)
       const char *bufl[1];
 
       bufl[0] = buf;
-      
+
 #define MKFILE_MODE (S_IRUSR | S_IWUSR | S_IWGRP | S_IRGRP | S_IROTH)
-      
+
       if (rtems_bsdnet_domain_name &&
           (strlen(rtems_bsdnet_domain_name) < (sizeof(buf) - 1))) {
         strcpy(buf, "search ");
@@ -1025,7 +1096,7 @@ dhcp_init (int update_files)
       }
     }
   }
-  
+
   /*
    * Configure the interface with the new settings
    */
@@ -1037,8 +1108,10 @@ dhcp_init (int update_files)
    */
   if (dhcp_lease_time != 0xffffffff)
     dhcp_start_task (sdl, &reply, 150);
-  
+
   soclose (so);
+
+  return 0;
 }
 
 /*
@@ -1048,11 +1121,35 @@ dhcp_init (int update_files)
  */
 void rtems_bsdnet_do_dhcp (void)
 {
+  bool update = true;
   rtems_bsdnet_semaphore_obtain ();
-  dhcp_init (TRUE);
+  while( dhcp_init (update) < 0 ) {
+    update = false;
+    rtems_bsdnet_semaphore_release();
+    rtems_task_wake_after(TOD_MILLISECONDS_TO_TICKS(1000));
+    rtems_bsdnet_semaphore_obtain ();
+  }
   rtems_bsdnet_semaphore_release ();
 }
 
+int rtems_bsdnet_do_dhcp_timeout( void )
+{
+  int return_value;
+
+  rtems_bsdnet_semaphore_obtain ();
+  return_value = dhcp_init (false);
+  rtems_bsdnet_semaphore_release ();
+
+  return return_value;
+}
+
+void rtems_bsdnet_dhcp_down (void)
+{
+   if(dhcp_task_id != 0) {
+     rtems_event_send (dhcp_task_id, RTEMS_EVENT_0);
+   }
+}
+
 void
 rtems_bsdnet_do_dhcp_refresh_only (unsigned long xid,
                                    unsigned long lease_time,
@@ -1074,7 +1171,7 @@ rtems_bsdnet_do_dhcp_refresh_only (unsigned long xid,
    */
   if (lease_time == 0xffffffff)
     return;
-  
+
   /*
    * Find a network interface.
    */
@@ -1092,17 +1189,21 @@ rtems_bsdnet_do_dhcp_refresh_only (unsigned long xid,
           }
         }
 
-  if (!match)
-    panic ("dhcpc: no matching interface");
+  if (!match) {
+    printf ("dhcpc: no matching interface\n");
+    return;
+  }
 
   for (ifa = mtif->if_addrlist; ifa != NULL; ifa = ifa->ifa_next)
     if (ifa->ifa_addr->sa_family == AF_LINK &&
         (sdl = ((struct sockaddr_dl *) ifa->ifa_addr)) &&
         sdl->sdl_type == IFT_ETHER)
       break;
-  
-  if (!match)
-    panic ("dhcpc: no matching interface");
+
+  if (!match) {
+    printf ("dhcpc: no matching interface address\n");
+    return;
+  }
 
   /*
    * Set up given values in a simulated DHCP reply.
@@ -1119,7 +1220,7 @@ rtems_bsdnet_do_dhcp_refresh_only (unsigned long xid,
 
   dhcp_lease_time = lease_time;
   dhcp_elapsed_time = elapsed_time;
-  
+
   if (hostname)
   {
     sethostname ((char *) hostname, strlen (hostname));
@@ -1128,6 +1229,3 @@ rtems_bsdnet_do_dhcp_refresh_only (unsigned long xid,
 
   dhcp_start_task (sdl, &reply, 150);
 }
-
-#undef D_COMPILING_BSD_KERNEL
-#undef KERNEL
