@@ -8,7 +8,7 @@
  *  found in the file LICENSE in this distribution or at
  *  http://www.rtems.com/license/LICENSE.
  *
- *  $Id: exshutdown.c,v 1.1 2007/05/09 18:27:26 joel Exp $
+ *  $Id$
  */
 
 #if HAVE_CONFIG_H
@@ -19,10 +19,6 @@
 #include <rtems/score/sysstate.h>
 #include <rtems/score/thread.h>
 
-#if defined(__USE_INIT_FINI__)
-#include <stdlib.h> /* for atexit() */
-#endif
-
 /*
  *  rtems_shutdown_executive
  *
@@ -39,12 +35,7 @@ void rtems_shutdown_executive(
    uint32_t   result
 )
 {
-  if ( _System_state_Current != SYSTEM_STATE_SHUTDOWN ) {
-
-    #if defined(__USE_INIT_FINI__)
-      extern void _fini( void );
-      atexit( _fini );
-    #endif
+  if ( !_System_state_Is_shutdown( _System_state_Get() ) ) {
     _System_state_Set( SYSTEM_STATE_SHUTDOWN );
     _Thread_Stop_multitasking();
   }
